#Makefile for SPRtools2
#x86 Linux version
#Call from sprtools/source2 directory

PROCESSOR = LINUX_ARM

CC      = g++
CFLAGS  = -c -D$(PROCESSOR) -O3 -Wno-unused-result
CCOUT   = -MMD -MP -c -o $@ $<
LINK    = $(CC)
LDFLAGS = 
LDLIBS  = -lm
AR      = ar
ARFLAGS = rvs

BIN     = ../bin2

TARGETS =				\
	$(BIN)/spr_fsi		\
	$(BIN)/spr_pal		\
	$(BIN)/spr_diff		\
	$(BIN)/spr_info

SOURCES = $(wildcard *.cpp)
OBJECTS = $(SOURCES:.cpp=.o)
DEPENDS = $(SOURCES:.cpp=.d)

all:	$(BIN)		\
		$(TARGETS)

clean:
	rm -f $(TARGETS)
	rm -f $(OBJECTS)
	rm -f $(DEPENDS)

-include  $(DEPENDS)    

%.o: %.cpp
	$(CC) $(CFLAGS) $(CCOUT)

sprlib.a: io.o sprite.o colours.o process.o palette.o
	$(AR) $(ARFLAGS) $@ $^

$(BIN):
	mkdir -p $@

$(BIN)/spr_fsi:  spr_fsi.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@
	
$(BIN)/spr_pal:  spr_pal.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr_diff:  spr_diff.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr3spr:  spr3spr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr2xbm:  spr2xbm.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr2bmp:  spr2bmp.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr2ras:  spr2ras.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr2sb:  spr2sb.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr2tga:  spr2tga.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr2xwd:  spr2xwd.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr2clr:  spr2clr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/bmp2spr:  bmp2spr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/ras2spr:  ras2spr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/sb2spr:  sb2spr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/tga2spr:  tga2spr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/xwd2spr:  xwd2spr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/clr2spr:  clr2spr.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/sb_info:  sb_info.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/bmp_info:  bmp_info.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/ras_info:  ras_info.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/xwd_info:  xwd_info.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/tga_info:  tga_info.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/clr_info:  clr_info.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/spr_info:  spr_info.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@

$(BIN)/chmode:  chmode.o sprlib.a
	$(LINK) $(LDFLAGS) $^ $(LDLIBS) -o $@
